#ifndef ORBIT_PROCESSOR_H
#define ORBIT_PROCESSOR_H

#include "controls.h"
#include "format.h"
#include "processor.h"

class OrbitProcessor : public Processor {
 public:
  OrbitProcessor(uint32_t num_orbits_per_packet, bool apply_zs, bool cmssw_headers,
                 uint16_t source_id, ctrl &control)
      : Processor(),
        nOrbitsPerPacket(num_orbits_per_packet),
        doZS(apply_zs),
        cmsswHeaders(cmssw_headers),
        source_id_(source_id),
        control_(control) {}

 protected:
  struct FillOrbitMetadata {
    uint32_t counts;
    uint32_t orbit;
    uint32_t filled_bxs;
  };

  bool HasTrailer(Slice &input, char *&rd_ptr) const;
  bool CheckFrameMultBlock(size_t inputSize) const;
  std::pair<uint32_t, bool> ProcessOrbitHeader(char *rd_ptr);
  size_t fillFRDEventHeader_V6(char *wr_ptr_FRDHead, uint32_t inputSize, uint32_t orbit) const;

  void ProcessSliceImpl(Slice &input, Slice &out) override;

  virtual FillOrbitMetadata FillOrbit(orbit_trailer *trailer, char *rd_ptr, char *wr_ptr,
                                      const char *rd_end_ptr, const char *wr_end_ptr) = 0;

  virtual uint32_t GetOrbitSizeInBytes(const FillOrbitMetadata &meta) const = 0;

  const uint32_t nOrbitsPerPacket;
  const bool doZS;
  const bool cmsswHeaders;
  const uint16_t source_id_;

  ctrl &control_;
};

#endif  // ORBIT_PROCESSOR_H